﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace HeuristicLab.Algorithms.Bandits.BanditPolicies {
  public class UCBNormalPolicy : IBanditPolicy {

    public int SelectAction(Random random, IEnumerable<IBanditPolicyActionInfo> actionInfos) {
      var myActionInfos = actionInfos.OfType<MeanAndVariancePolicyActionInfo>();
      int bestAction = -1;
      double bestQ = double.NegativeInfinity;
      int totalTries = myActionInfos.Where(a => !a.Disabled).Sum(a => a.Tries);
      int aIdx = -1;
      foreach (var aInfo in myActionInfos) {
        aIdx++;
        if (aInfo.Disabled) continue;
        if (totalTries <= 1 || aInfo.Tries <= 1 || aInfo.Tries <= Math.Ceiling(8 * Math.Log(totalTries))) return aIdx;

        var tries = aInfo.Tries;
        var avgReward = aInfo.AvgReward;
        var rewardVariance = aInfo.RewardVariance;
        var estVariance = 16.0 * rewardVariance * (Math.Log(totalTries - 1) / tries);
        var q = avgReward + Math.Sqrt(estVariance);
        if (q > bestQ) {
          bestQ = q;
          bestAction = aIdx;
        }
      }
      Debug.Assert(bestAction > -1);
      return bestAction;
    }

    public IBanditPolicyActionInfo CreateActionInfo() {
      return new MeanAndVariancePolicyActionInfo();
    }

    public override string ToString() {
      return "UCBNormalPolicy";
    }
  }
}
